@@ -1,42 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Copyright (C) 2016 Intel Corporation.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtCore module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// Copyright (C) 2016 Intel Corporation.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #ifdef QT_NO_DEBUG
 #undef QT_NO_DEBUG
@@ -50,13 +14,10 @@
 #include "qmetaobject.h"
 #include <private/qtextstream_p.h>
 #include <private/qtools_p.h>
-#include <ctype.h>
 
 QT_BEGIN_NAMESPACE
 
-using QtMiscUtils::toHexUpper;
-using QtMiscUtils::toHexLower;
-using QtMiscUtils::fromHex;
+using namespace QtMiscUtils;
 
 /*
     Returns a human readable representation of the first \a maxSize
@@ -71,7 +32,7 @@ QByteArray QtDebugUtils::toPrintable(const char *data, qint64 len, qsizetype max
     QByteArray out;
     for (qsizetype i = 0; i < qMin(len, maxSize); ++i) {
         char c = data[i];
-        if (isprint(c)) {
+        if (isAsciiPrintable(c)) {
             out += c;
         } else {
             switch (c) {
@@ -197,7 +158,7 @@ QByteArray QtDebugUtils::toPrintable(const char *data, qint64 len, qsizetype max
 QDebug::~QDebug()
 {
     if (stream && !--stream->ref) {
-        if (stream->space && stream->buffer.endsWith(QLatin1Char(' ')))
+        if (stream->space && stream->buffer.endsWith(u' '))
             stream->buffer.chop(1);
         if (stream->message_output) {
             qt_message_output(stream->type,
@@ -223,7 +184,7 @@ void QDebug::putUcs4(uint ucs4)
             stream->ts << "\\u" << qSetFieldWidth(4);
         else
             stream->ts << "\\U" << qSetFieldWidth(8);
-        stream->ts << Qt::hex << qSetPadChar(QLatin1Char('0')) << ucs4 << Qt::reset;
+        stream->ts << Qt::hex << qSetPadChar(u'0') << ucs4 << Qt::reset;
     }
     maybeQuote('\'');
 }
@@ -231,17 +192,15 @@ void QDebug::putUcs4(uint ucs4)
 // These two functions return true if the character should be printed by QDebug.
 // For QByteArray, this is technically identical to US-ASCII isprint();
 // for QString, we use QChar::isPrint, which requires a full UCS-4 decode.
-static inline bool isPrintable(uint ucs4)
-{ return QChar::isPrint(ucs4); }
-static inline bool isPrintable(ushort uc)
-{ return QChar::isPrint(uc); }
+static inline bool isPrintable(char32_t ucs4) { return QChar::isPrint(ucs4); }
+static inline bool isPrintable(char16_t uc) { return QChar::isPrint(uc); }
 static inline bool isPrintable(uchar c)
-{ return c >= ' ' && c < 0x7f; }
+{ return isAsciiPrintable(c); }
 
 template <typename Char>
 static inline void putEscapedString(QTextStreamPrivate *d, const Char *begin, size_t length, bool isUnicode = true)
 {
-    QChar quote(QLatin1Char('"'));
+    QChar quote(u'"');
     d->write(&quote, 1);
 
     bool lastWasHexEscape = false;
@@ -251,7 +210,7 @@ static inline void putEscapedString(QTextStreamPrivate *d, const Char *begin, si
         if (Q_UNLIKELY(lastWasHexEscape)) {
             if (fromHex(*p) != -1) {
                 // yes, insert it
-                QChar quotes[] = { QLatin1Char('"'), QLatin1Char('"') };
+                QChar quotes[] = { quote, quote };
                 d->write(quotes, 2);
             }
             lastWasHexEscape = false;
@@ -276,7 +235,7 @@ static inline void putEscapedString(QTextStreamPrivate *d, const Char *begin, si
 
         // print as an escape sequence (maybe, see below for surrogate pairs)
         qsizetype buflen = 2;
-        ushort buf[sizeof "\\U12345678" - 1];
+        char16_t buf[std::char_traits<char>::length("\\U12345678")];
         buf[0] = '\\';
 
         switch (*p) {
@@ -312,7 +271,7 @@ static inline void putEscapedString(QTextStreamPrivate *d, const Char *begin, si
             if (QChar::isHighSurrogate(*p)) {
                 if ((p + 1) != end && QChar::isLowSurrogate(p[1])) {
                     // properly-paired surrogates
-                    uint ucs4 = QChar::surrogateToUcs4(*p, p[1]);
+                    char32_t ucs4 = QChar::surrogateToUcs4(*p, p[1]);
                     if (isPrintable(ucs4)) {
                         buf[0] = *p;
                         buf[1] = p[1];
@@ -335,8 +294,8 @@ static inline void putEscapedString(QTextStreamPrivate *d, const Char *begin, si
                 // improperly-paired surrogates, fall through
             }
             buf[1] = 'u';
-            buf[2] = toHexUpper(ushort(*p) >> 12);
-            buf[3] = toHexUpper(ushort(*p) >> 8);
+            buf[2] = toHexUpper(char16_t(*p) >> 12);
+            buf[3] = toHexUpper(char16_t(*p) >> 8);
             buf[4] = toHexUpper(*p >> 4);
             buf[5] = toHexUpper(*p);
             buflen = 6;
@@ -361,7 +320,7 @@ void QDebug::putString(const QChar *begin, size_t length)
         // we'll reset the QTextStream formatting mechanisms, so save the state
         QDebugStateSaver saver(*this);
         stream->ts.d_ptr->params.reset();
-        putEscapedString(stream->ts.d_ptr.data(), reinterpret_cast<const ushort *>(begin), length);
+        putEscapedString(stream->ts.d_ptr.data(), reinterpret_cast<const char16_t *>(begin), length);
     }
 }
 
@@ -730,7 +689,7 @@ QDebug &QDebug::resetFormat()
 */
 
 /*!
-    \fn QDebug &QDebug::operator<<(QLatin1String t)
+    \fn QDebug &QDebug::operator<<(QLatin1StringView t)
 
     Writes the string, \a t, to the stream and returns a reference to the
     stream. Normally, QDebug prints the string inside quotes and transforms
@@ -799,6 +758,24 @@ QDebug &QDebug::resetFormat()
     \internal
 */
 
+/*!
+    \since 6.5
+    \fn template <typename Char, typename...Args> QDebug &QDebug::operator<<(const std::basic_string<Char, Args...> &s)
+    \fn template <typename Char, typename...Args> QDebug &QDebug::operator<<(std::basic_string_view<Char, Args...> s)
+
+    Writes the string or string-view \a s to the stream and returns a reference
+    to the stream.
+
+    These operators only participate in overload resolution if \c Char is one of
+    \list
+    \li char
+    \li char8_t (C++20 only)
+    \li char16_t
+    \li char32_t
+    \li wchar_t
+    \endlist
+*/
+
 /*!
     \fn template <class T> QString QDebug::toString(T &&object)
     \since 6.0
@@ -814,6 +791,15 @@ QDebug &QDebug::resetFormat()
     support streaming into QDebug.
 */
 
+/*!
+    \fn template <class T, qsizetype P> QDebug operator<<(QDebug debug, const QVarLengthArray<T,P> &array)
+    \relates QDebug
+    \since 6.3
+
+    Writes the contents of \a array to \a debug. \c T needs to
+    support streaming into QDebug.
+*/
+
 /*!
     \fn template <typename T, typename Alloc> QDebug operator<<(QDebug debug, const std::list<T, Alloc> &vec)
     \relates QDebug
@@ -890,14 +876,6 @@ QDebug &QDebug::resetFormat()
     \c T need to support streaming into QDebug.
 */
 
-/*!
-    \fn template <class T1, class T2> QDebug operator<<(QDebug debug, const QPair<T1, T2> &pair)
-    \relates QDebug
-
-    Writes the contents of \a pair to \a debug. Both \c T1 and
-    \c T2 need to support streaming into QDebug.
-*/
-
 /*!
     \fn template <class T1, class T2> QDebug operator<<(QDebug debug, const std::pair<T1, T2> &pair)
     \relates QDebug
@@ -956,7 +934,7 @@ QDebug &QDebug::resetFormat()
 
     QDebugStateSaver is typically used in the implementation of an operator<<() for debugging:
 
-    \snippet tools/customtype/message.cpp custom type streaming operator
+    \snippet customtype/customtypeexample.cpp custom type streaming operator
 
     \since 5.1
 */
@@ -976,7 +954,7 @@ public:
     {
         const bool currentSpaces = m_stream->space;
         if (currentSpaces && !m_spaces)
-            if (m_stream->buffer.endsWith(QLatin1Char(' ')))
+            if (m_stream->buffer.endsWith(u' '))
                 m_stream->buffer.chop(1);
 
         m_stream->space = m_spaces;
